/*************************************************** */
/* Rule Set Based Access Control                     */
/*                                                   */
/* Author and (c) 1999-2005: Amon Ott <ao@rsbac.org> */
/*                                                   */
/* Last modified: 31/May/2005                        */
/*************************************************** */

#include <stdio.h>
#include <stdlib.h>
#include <rsbac/types.h>
#include <rsbac/syscalls.h>
#include <rsbac/error.h>
#include <rsbac/getname.h>
#include "nls.h"

int main(int argc, char ** argv)
{
  int res = 0;
  rsbac_rc_role_id_t role;

 locale_init();

  res = rsbac_rc_get_current_role(&role);
      if(res<0)
        {
          error_exit(res);
        }
      else
        {
          printf(gettext("%s: current role is %u\n"), argv[0], role);
        }
    
  return (res);
}
