/*************************************************** */
/* Rule Set Based Access Control                     */
/* Author and (c) 1999-2005: Amon Ott <ao@rsbac.org> */
/* Generic lists - internal structures               */
/* Last modified: 04/Apr/2005                        */
/*************************************************** */

#ifndef __RSBAC_REPL_LISTS_H
#define __RSBAC_REPL_LISTS_H

#include <rsbac/repl_types.h>

#define RSBAC_LIST_REPL_PROC_NAME "repl_lists"
#define RSBAC_LIST_REPL_PARTNER_VERSION 1
#define RSBAC_LIST_REPL_PARTNER_KEY 0x3632f7ae
#define RSBAC_LIST_REPL_PARTNER_FILENAME "replpar"

#endif
