#!/bin/sh
attr_set_user root security_level 3
attr_set_user root system_role 2
attr_set_user root pm_role 4
attr_set_user root rc_def_role 2
attr_set_user bin system_role 2
attr_set_user secoff system_role 1
attr_set_user secoff pm_role 1
attr_set_user secoff rc_def_role 1
attr_set_user dataprot pm_role 2
attr_set_user tpmanager pm_role 3
