/********************************** */
/* Rule Set Based Access Control    */
/* Author and (c) 1999-2005:        */
/*   Amon Ott <ao@rsbac.org>        */
/* Getname functions for JAIL module */
/* Last modified: 27/May/2005       */
/********************************** */

#ifndef __RSBAC_JAIL_GETNAME_H
#define __RSBAC_JAIL_GETNAME_H

void rsbac_jail_log_missing_cap(int cap);

#endif
