#include <stdio.h>
#include <stdlib.h>
#include <linux/kdev_t.h>
#include <rsbac/types.h>
#include <rsbac/syscalls.h>
#include <rsbac/getname.h>
#include <rsbac/error.h>

int main(int argc, char ** argv)
{
  int res = 0;
  int i;
  struct rsbac_auth_cap_range_t cap_range;

  if(argc > 1)
    {
      cap_range.first = 0;
      cap_range.last = 0;
      rsbac_auth_add_p_cap(strtoul(argv[1],0,0),
                           ACT_real,
                           cap_range,
                           3600);
      cap_range.first = 400;
      cap_range.last = 400;
      rsbac_auth_add_p_cap(strtoul(argv[1],0,0),
                           ACT_real,
                           cap_range,
                           3600);
    }
  exit(0);
}
